@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1999-2000 Image Power, Inc. and the University of
  *   British Columbia.
- * Copyright (c) 2001 Michael David Adams.
+ * Copyright (c) 2001-2002 Michael David Adams.
  * All rights reserved.
  */
 
@@ -12,7 +12,7 @@
  * IMAGE POWER JPEG-2000 PUBLIC LICENSE
  * ************************************
  * 
- * WHEREAS:
+ * GRANT:
  * 
  * Permission is hereby granted, free of charge, to any person (the "User")
  * obtaining a copy of this software and associated documentation, to deal
@@ -119,10 +119,6 @@
 #ifndef JAS_STREAM_H
 #define JAS_STREAM_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 /******************************************************************************\
 * Includes.
 \******************************************************************************/
@@ -135,6 +131,10 @@ extern "C" {
 #endif
 #include <jasper/jas_types.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /******************************************************************************\
 * Constants.
 \******************************************************************************/
@@ -159,6 +159,8 @@ extern "C" {
 /* The stream should be created/truncated. */
 #define JAS_STREAM_CREATE	0x0010
 
+#define JAS_STREAM_NOCLOSE	0x0020
+
 /*
  * Stream buffering flags.
  */
